<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:template="http://www.boddie.org.uk/ns/xmltools/template"
                version="1.0">

  <xsl:param name="database"/>

  <xsl:template match="configuration">
    <xsl:copy>
      <xsl:apply-templates select="@*"/>
      <!-- Complete all multiple choice fields. -->
      <details>
        <xsl:call-template name="complete-element">
          <xsl:with-param name="found-elements" select="details/base-system"/>
          <xsl:with-param name="element-name">base-system</xsl:with-param>
        </xsl:call-template>
        <xsl:call-template name="complete-element">
          <xsl:with-param name="found-elements" select="details/cpu"/>
          <xsl:with-param name="element-name">cpu</xsl:with-param>
        </xsl:call-template>
      </details>
      <memory>
        <xsl:for-each select="memory/memory-unit">
          <xsl:call-template name="complete-element">
            <xsl:with-param name="found-elements" select="."/>
            <xsl:with-param name="element-name">memory-unit</xsl:with-param>
          </xsl:call-template>
        </xsl:for-each>
      </memory>
      <hard-disks>
        <xsl:for-each select="hard-disks/hard-disk">
          <xsl:call-template name="complete-element">
            <xsl:with-param name="found-elements" select="."/>
            <xsl:with-param name="element-name">hard-disk</xsl:with-param>
          </xsl:call-template>
        </xsl:for-each>
      </hard-disks>
      <storage>
        <xsl:for-each select="storage/storage-unit">
          <xsl:call-template name="complete-element">
            <xsl:with-param name="found-elements" select="."/>
            <xsl:with-param name="element-name">storage-unit</xsl:with-param>
          </xsl:call-template>
        </xsl:for-each>
      </storage>
      <peripherals>
        <xsl:call-template name="complete-element">
          <xsl:with-param name="found-elements" select="peripherals/keyboard"/>
          <xsl:with-param name="element-name">keyboard</xsl:with-param>
        </xsl:call-template>
        <xsl:call-template name="complete-element">
          <xsl:with-param name="found-elements" select="peripherals/mouse"/>
          <xsl:with-param name="element-name">mouse</xsl:with-param>
        </xsl:call-template>
        <xsl:call-template name="complete-element">
          <xsl:with-param name="found-elements" select="peripherals/screen"/>
          <xsl:with-param name="element-name">screen</xsl:with-param>
        </xsl:call-template>
      </peripherals>
    </xsl:copy>
  </xsl:template>

  <xsl:template name="complete-element">
    <xsl:param name="found-elements"/>
    <xsl:param name="element-name"/>
    <xsl:variable name="element-enum-name"><xsl:value-of select="$element-name"/>-enum</xsl:variable>
    <xsl:variable name="has-set-values" select="$found-elements[@value]"/>
    <!-- Make the element with the different values. -->
    <xsl:element name="{$element-name}">
      <!-- Find the values in the database. -->
      <xsl:for-each select="$database/database/*[local-name() = $element-name]/*[local-name() = $element-enum-name]">
        <xsl:variable name="enum-element" select="."/>
        <!-- Copy each option. -->
        <xsl:copy>
          <!-- Only include the value, adding the price only if the item is selected. -->
          <xsl:apply-templates select="@value"/>
          <!-- Find whether this item is selected. -->
          <xsl:choose>
            <!-- In the current field, the value attribute matches this value. -->
            <xsl:when test="$found-elements[@value = $enum-element/@value]">
              <xsl:attribute name="value-is-set">true</xsl:attribute>
              <xsl:apply-templates select="@price"/>
            </xsl:when>
            <!-- This value is the first and no value attribute is specified. -->
            <xsl:when test="not($has-set-values) and count($enum-element/preceding-sibling::*) = 0">
              <xsl:attribute name="value-is-set">true</xsl:attribute>
              <xsl:apply-templates select="@price"/>
            </xsl:when>
            <xsl:otherwise/>
          </xsl:choose>
        </xsl:copy>
      </xsl:for-each>
    </xsl:element>
  </xsl:template>

  <!-- Replicate unknown elements. -->

  <xsl:template match="@*|node()">
    <xsl:copy>
      <xsl:apply-templates select="@*|node()"/>
    </xsl:copy>
  </xsl:template>

</xsl:stylesheet>
